/***************************************************************************
 *
 * searchlog.c - Log the progress of a UFN of form search.
 * 
 ****************************************************************************/

#include "defs.h"
#include "searchlog.h"

#include <X11/Xatom.h>
#include <Xm/Form.h>
#include <Xm/Text.h>
#include <Xm/PushB.h>
#include <Xm/ScrolledW.h>
#include <Xm/List.h>

#include "query.h"
#include "ufsearch.h"
#include "quipu/common.h"
#include "windows.h"
#include "callbacks.h"
#include "objects.h"
#include "requests.h"
#include "configure.h"
#include "xutil.h"
#include "y.tab.h"

/**************************************************
 *
 * Local data.
 * 
 */

/* Context data for log window. */

typedef struct LogContext
{
  entryList dnlist;
  int reqid;
  QyErrorList errors;
} *LogContext;

/*****************************************************************************
 * LogFormReport()
 *
 * Log a search report to the associated log window.
 *
 ****************************************************************************/

void LogFormReport(objid, reqid)
     int objid;
     int reqid;
{
  Widget logwin, listbox;
  QyFormSearchReport report;
  entryList currdn;
  int count, c;
  XmString *stringarr, *liststr;
  XtPointer ptr;
  LogContext logctx;
  Widget *instances;
  
  report = QyGetFormReport(reqid);
  if (report == (QyFormSearchReport) NULL)
    return;

  /* Get widget for report primitive. */
  instances = XLUgetPrimitiveList(objid, SEARCH_LOG);
  if (instances == (Widget *) NULL)
    return;

  logwin = instances[0];
  XtFree((char *) instances);

  listbox = XtNameToWidget(logwin, "logProgressList");

  /* Get and free old report structure. */
  ptr = XLUgetPrimitiveData(listbox);
  logctx = (LogContext) ptr;
  if (logctx != (LogContext) NULL)
    {
      dn_list_free(&logctx->dnlist);
    }
  else
    {
      logctx = (LogContext) XtNew(struct LogContext);
      logctx->errors = NULLError;

      XLUsavePrimitiveData(listbox, (XtPointer) logctx, LogWindowCleanUp);
    }

  /* Get and set of matched entries. */
  dn_list_copy(report->srchlist, &logctx->dnlist);
  
  switch (report->type)
    {
    case QyFormMatches:
    case QyFormAttempt:

      /* Reset displayed search list. */
      XmListDeleteAllItems(listbox);

      count = dn_list_count(report->srchlist);
      stringarr = (XmString *) XtMalloc((unsigned) (sizeof(XmString) * count));

      for (currdn = logctx->dnlist, liststr = stringarr;
	   currdn != NULLEntryList;
	   currdn = currdn->next, liststr++)
	{
	  char UFName[LINESIZE];

	  QyPrintFriendlyDN(currdn->string_dn, UFName);
	  *liststr = XmStringCreateSimple(UFName);
	}

      XmListAddItems(listbox, stringarr, count, 1);

      for (c = 0, liststr = stringarr; c < count; c++, liststr++)
	XmStringFree(*liststr);

      (void) XtFree((XtPointer) stringarr);
      break;
    }

  QyFreeFormReport(&report);
} /* LogFormReport */

/*****************************************************************************
 * LogFormCompletion()
 *
 * A form search has completed. 
 *
 ****************************************************************************/

/* ARGSUSED */
void LogFormCompletion(reqid, errors)
     int reqid;
     QyErrorList errors;
{
} /* LogFormCompletion */

/*****************************************************************************
 * LogUFNReport()
 *
 * Log an intermediate UFN search report.
 *
 ****************************************************************************/

void LogUFNReport(objid, reqid)
     int objid;
     int reqid;
{
  Widget logwin, listbox;
  QyUFNReport report;
  entryList currdn;
  int count, c;
  XmString *stringarr, *liststr;
  XtPointer ptr;
  LogContext logctx;
  Widget *instances;
  
  report = QyGetUFNReport(reqid);
  if (report == (QyUFNReport) NULL)
    return;

  /* Get widget for report primitive. */
  instances = XLUgetPrimitiveList(objid, SEARCH_LOG);
  if (instances == (Widget *) NULL)
    return;

  logwin = instances[0];
  XtFree((char *) instances);

  listbox = XtNameToWidget(logwin, "logProgressList");

  /* Get and free old report structure. */
  ptr = XLUgetPrimitiveData(listbox);
  logctx = (LogContext) ptr;
  if (logctx != (LogContext) NULL)
    {
      dn_list_free(&logctx->dnlist);
    }
  else
    {
      logctx = (LogContext) XtNew(struct LogContext);
      logctx->errors = NULLError;

      XLUsavePrimitiveData(listbox, (XtPointer) logctx, LogWindowCleanUp);
    }

  /* Get and set current report. */
  dn_list_copy(report->srchlist, &logctx->dnlist);
  
  /* Reset displayed search list. */
  XmListDeleteAllItems(listbox);

  count = dn_list_count(report->srchlist);
  stringarr = (XmString *) XtMalloc((unsigned) (sizeof(XmString) * count));
  
  for (currdn = logctx->dnlist, liststr = stringarr;
       currdn != NULLEntryList;
       currdn = currdn->next, liststr++)
    {
      char UFName[LINESIZE];
      
      QyPrintFriendlyDN(currdn->string_dn, UFName);
      *liststr = XmStringCreateSimple(UFName);
    }
  
  XmListAddItems(listbox, stringarr, count, 1);
  
  for (c = 0, liststr = stringarr; c < count; c++, liststr++)
    XmStringFree(*liststr);
  
  (void) XtFree((XtPointer) stringarr);
  QyFreeUFNReport(&report);
} /* LogUFNReport */

/*****************************************************************************
 * LogUFNCompletion()
 *
 * Log the completion of a UFN search.
 *
 ****************************************************************************/

/* ARGSUSED */
void LogUFNCompletion()
{
} /* LogUFNCompletion */

/*****************************************************************************
 * ItemSelected()
 *
 * An entry has been selected from the list box. Enable the
 * "Prune" button.
 *
 ****************************************************************************/

/* ARGSUSED */
void ItemSelected(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  Widget lbox;
  int *listposlist, count, listindex;
  XtPointer ptr;
  LogContext logctx;
  entryList currdn;
  int objid;

  objid = XLUgetWindowObjectId(w);

  /* Get list of displayed entries stored in current report structure */
  lbox = w;
  ptr = XLUgetPrimitiveData(lbox);
  if (ptr == (XtPointer) NULL)
    return;

  logctx = (LogContext) ptr;
  
  /* Get the selected item index */
  count = 0;
  if (XmListGetSelectedPos(lbox, &listposlist, &count) == FALSE || count != 1)
    {
      XtFree((XtPointer) listposlist);
      return;
    }

  listindex = listposlist[0];
  XtFree((XtPointer) listposlist);

  /* Get DN of selected entry. */
  currdn = logctx->dnlist;
  for (count = 1; count < listindex; count++)
    currdn = currdn->next;

  /* Set DN as window\'s current entry */
  XLUsetWindowEntry(objid, currdn);
} /* ItemSelected */

/*****************************************************************************
 * LogWindowCleanUp()
 *
 * Free context data for log window.
 *
 ****************************************************************************/

/* ARGSUSED */
void LogWindowCleanUp(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  LogContext logctx;

  if (closure == (XtPointer) NULL)
    return;

  logctx = (LogContext) closure;

  dn_list_free(&logctx->dnlist);
  QyFreeErrorList(&logctx->errors);

  XtFree((char *) logctx);
} /* LogWindowCleanUp */

/*****************************************************************************
 * XLUpruneSearch()
 *
 * Prune UFN and form searches.
 *
 ****************************************************************************/

/* ARGSUSED */
void XLUpruneSearch(w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     int num_params;
{
  XLUrequestRecord record;
  XLUwindowObject winobj;
  Widget lbox;
  int *listposlist, count, listindex, objid;
  XtPointer ptr;
  LogContext logctx;
  QyRequestState state;
  XLUactionList actlist;

  extern XLUrequestRecord XLUrequestList;

  objid = XLUgetWindowObjectId(w);
  winobj = XLUgetWindowObject(objid);

  if (winobj->current_entry == NULLEntryList)
    /* Nothing to prune! */
    return;
  
  /* Find requests owned by this window object */
  count = 0;
  for (record = XLUrequestList;
       record != NULLXLUrequestRecord;
       record = record->next)
    {
      if (record->objid == objid)
	/* Got a request associated with this window */
	{
	  switch (record->type)
	    {
	    case UFSEARCH:
	      /* Prune a form search */
	      count++;

	      state = QyFormPruneSearch
		(winobj->current_entry->string_dn, record->reqid);

	      switch (state)
		{
		case RQ_results_returned:

		  actlist = record->actlist;
		  (*record->result_cb)(record->reqid,
				       objid,
				       record->action_id,
				       actlist);

		  doPrimitiveSensitivity(objid);
		  callActionAux(w,
				objid,
				SEARCH_LOG,
				record->action_id,
				actlist? actlist->next : NULLActionList);
      
		  XLUrequestComplete(record->reqid);
		  break;

		case RQ_report_returned:
		  
		  (*record->report_cb)(record->reqid, objid);
		  break;
		}

	      break;

	    case UFNAME:

	      count++;

	      /* Prune UFN search */
	      state = QyUFNPruneSearch
		(winobj->current_entry->string_dn, record->reqid);

	      switch (state)
		{
		case RQ_results_returned:

		  actlist = record->actlist;
		  (*record->result_cb)(record->reqid,
				       objid,
				       record->action_id,
				       actlist);

		  doPrimitiveSensitivity(objid);
		  callActionAux(w,
				objid,
				SEARCH_LOG,
				record->action_id,
				actlist? actlist->next : NULLActionList);
      
		  XLUrequestComplete(record->reqid);
		  break;

		case RQ_report_returned:
		  
		  (*record->report_cb)(record->reqid, objid);
		  break;
		}

	      break;

	    default:
	      /* Not a search request, do nothing */
	      break;
	    }
	}
    }

  if (count == 0)
    /* Didn\'t prune anything, so return */
    return;

  /* Get list of entries stored in current report structure */
  lbox = w;
  ptr = XLUgetPrimitiveData(lbox);
  if (ptr == (XtPointer) NULL)
    /* Hmmm */
    return;

  logctx = (LogContext) ptr;

  /* Remove pruned entry from log list */
  dn_list_remove(&logctx->dnlist, winobj->current_entry->string_dn);

  /* Entry pruned and deleted from list, so delete from "current_entry" */
  XLUsetWindowEntry(objid, NULLEntryList);
  
  /* Get the selected item index */
  count = 0;
  if (XmListGetSelectedPos(lbox, &listposlist, &count) == FALSE || count != 1)
    {
      XtFree((XtPointer) listposlist);
      return;
    }

  /* Remove it from the log list box */
  listindex = listposlist[0];
  XtFree((XtPointer) listposlist);

  XmListDeletePos(lbox, listindex);
} /* XLUpruneSearch */

